require "spec_helper"

describe Opensips::Pi do
  subject { Opensips::Pi }

  let(:user) { "fulano" }
  let(:domain) { "domain" }
  let(:pass) { "senha" }
  let(:bin) { "opensipsctl-bin" }

  before do
    subject.bin = nil
    subject.logger = nil
    Logger.any_instance.stub(:info => double, :error => double)
  end

  describe ".bin" do
    it "returns opensipsctl by default" do
      subject.bin = nil
      subject.bin.should be == "opensipsctl"
    end

    it "returns a custom command" do
      subject.bin = "/usr/sbin/opensipsctl"
      subject.bin.should be == "/usr/sbin/opensipsctl"
    end
  end

  describe ".logger" do
    it "returns a new instance of Logger by default" do
      subject.logger = nil
      subject.logger.should be_kind_of Logger
    end

    it "returns a custom logger" do
      dummy = Class.new
      subject.logger = dummy
      subject.logger.should be == dummy
    end
  end

  describe ".add" do
    before { subject.stub(:bin => bin) }

    it "calls opensipsctl command with given username, domain and password" do
      Open3.should_receive(:capture3).with("#{bin} add #{user}@#{domain} #{pass}").and_return(["foo", "bar", double(:"success?" => true)])

      expect {
        subject.add(user, pass, domain)
      }.to_not raise_error
    end

    it "calls opensipsctl command with given username and password" do
      Open3.should_receive(:capture3).with("#{bin} add #{user} #{pass}").and_return(["foo", "bar", double(:"success?" => true)])

      expect {
        subject.add(user, pass)
      }.to_not raise_error
    end

    it "returns false when opensipsctl exit with 1" do
      Open3.stub(:capture3 => ["foo", "bar", double(:"success?" => false)])
      subject.add(user, pass).should == [false,"foo","bar"]
    end

    it "returns true when opensipsctl exit with 0" do
      Open3.stub(:capture3 => ["foo", "bar", double(:"success?" => true)])
      subject.add(user, pass).should be_true
    end

    it "escape arguments when calling opensipsctl" do
      user = "user with spaces"
      Open3.should_receive(:capture3).with("#{bin} add user\\ with\\ spaces #{pass}").and_return(["","",double(:"success?" => true)])

      subject.add(user, pass)
    end

    it_behaves_like "a method that log stdout and stderr messages", :add, "fulano", "senha"
  end

  describe ".remove" do
    before { subject.stub(:bin => bin) }

    it "calls opensipsctl command with given username and domain" do
      Open3.should_receive(:capture3).with("#{bin} rm #{user}@#{domain}").and_return(["foo", "bar", double(:"success?" => true)])

      expect {
        subject.remove(user, domain)
      }.to_not raise_error
    end

    it "calls opensipsctl command with given username" do
      Open3.should_receive(:capture3).with("#{bin} rm #{user}").and_return(["foo", "bar", double(:"success?" => true)])

      expect {
        subject.remove(user)
      }.to_not raise_error
    end

    it "returns false when opensipsctl exit with 1" do
      Open3.stub(:capture3 => ["foo", "bar", double(:"success?" => false)])
      subject.remove(user).should == [false,"foo","bar"]
    end

    it "returns true when opensipsctl exit with 0" do
      Open3.stub(:capture3 => ["foo", "bar", double(:"success?" => true)])
      subject.remove(user).should be_true
    end

    it "escape arguments when calling opensipsctl" do
      user = "user with spaces"
      Open3.should_receive(:capture3).with("#{bin} rm user\\ with\\ spaces").and_return(["","",double(:"success?" => true)])

      subject.remove(user)
    end

    it_behaves_like "a method that log stdout and stderr messages", :remove, "fulano"
  end

  describe ".password" do
    before { subject.stub(:bin => bin) }

    it "calls opensipsctl command with given username, password and domain" do
      Open3.should_receive(:capture3).with("#{bin} passwd #{user}@#{domain} #{pass}").and_return(["foo", "bar", double(:"success?" => true)])

      expect {
        subject.password(user, pass, domain)
      }.to_not raise_error
    end

    it "calls opensipsctl command with username and password" do
      Open3.should_receive(:capture3).with("#{bin} passwd #{user} #{pass}").and_return(["foo", "bar", double(:"success?" => true)])

      expect {
        subject.password(user, pass)
      }.to_not raise_error
    end

    it "returns false when opensipsctl exit with 1" do
      Open3.stub(:capture3 => ["foo", "bar", double(:"success?" => false)])
      subject.password(user, pass).should == [false,"foo","bar"]
    end

    it "returns true when opensipsctl exit with 0" do
      Open3.stub(:capture3 => ["foo", "bar", double(:"success?" => true)])
      subject.password(user, pass).should be_true
    end

    it "escape arguments when calling opensipsctl" do
      user = "user with spaces"
      Open3.should_receive(:capture3).with("#{bin} passwd user\\ with\\ spaces #{pass}").and_return(["","",double(:"success?" => true)])

      subject.password(user, pass)
    end

    it_behaves_like "a method that log stdout and stderr messages", :password, "fulano", "senha"
  end
end
